"""Recommended values related to security controls"""
from django.core.exceptions import ImproperlyConfigured
from django.conf import settings

from .utils import (
    global_from_env,
    get_env,
    get_env_value,
    configure_authentication_backends,
)

# Lockout users based on their username and IP address
AXES_LOCK_OUT_BY_COMBINATION_USER_AND_IP = True

# Log unsuccessful logins
AXES_ENABLE_ACCESS_FAILURE_LOG = True

# After 3 failed login attempts, lock out the combination (IP + user).
AXES_FAILURE_LIMIT = 3

# After 30 minutes, locked out accounts are automatically unlocked.
AXES_COOLOFF_TIME = 0.5

# Reverse proxy configuration
AXES_PROXY_COUNT = 1  # (Behind 1 proxy)
AXES_META_PRECEDENCE_ORDER = [
    "HTTP_X_FORWARDED_FOR",
    "REMOTE_ADDR",
]

# Configure the identity provider if the `{PREFIX}OAUTH_PROVIDER`
# `environment variable is set.
auth_config = get_env(
    OAUTH_PROVIDER=(str, ""),
    OAUTH_APP_CLIENT_ID=(str, ""),
    OAUTH_APP_CLIENT_SECRET=(str, ""),
    OAUTH_MICROSOFT_TENANT=(str, "common"),
)

if get_env_value(auth_config, "OAUTH_PROVIDER") == "microsoft":
    provider = get_env_value(auth_config, "OAUTH_PROVIDER")
    client_id = get_env_value(auth_config, "OAUTH_APP_CLIENT_ID")
    client_secret = get_env_value(auth_config, "OAUTH_APP_CLIENT_SECRET")
    tenant = get_env_value(auth_config, "OAUTH_MICROSOFT_TENANT")

    if client_id == "":
        raise ImproperlyConfigured("settings.OAUTH_APP_CLIENT_ID is required.")

    if client_secret == "":
        raise ImproperlyConfigured("settings.OAUTH_APP_CLIENT_SECRET is required.")

    AUTHLIB_OAUTH_CLIENTS = {
        f"{provider}": {
            "client_id": client_id,
            "client_secret": client_secret,
            "server_metadata_url": f"https://login.microsoftonline.com/{tenant}/v2.0/.well-known/openid-configuration",  # pylint: disable=line-too-long  # noqa: E501
            "client_kwargs": {
                "scope": "openid email profile",
            },
        }
    }

    PHAC_ASPC_OAUTH_USE_BACKEND = getattr(
        settings,
        "PHAC_ASPC_OAUTH_USE_BACKEND",
        "phac_aspc.django.helpers.auth.backend.PhacAspcOAuthBackend",
    )
    settings.PHAC_ASPC_OAUTH_USE_BACKEND = PHAC_ASPC_OAUTH_USE_BACKEND
    PHAC_ASPC_HELPER_OAUTH_PROVIDER = provider

#  AC-7 Automatic lockout of users after invalid login attempts
AUTHENTICATION_BACKENDS = configure_authentication_backends(
    [
        "django.contrib.auth.backends.ModelBackend",
    ]
)

#  AC-11 - Session controls
global_from_env(
    # Sessions expire in 20 minutes
    SESSION_COOKIE_AGE=(int, 1200),
    # Use HTTPS for session cookie
    SESSION_COOKIE_SECURE=(bool, True),
    # Sessions close when browser is closed
    SESSION_EXPIRE_AT_BROWSER_CLOSE=(bool, True),
    # Every requests extends the session (This is required for the WET session
    # plugin to function properly.)
    SESSION_SAVE_EVERY_REQUEST=(bool, True),
)



# flake8: noqa
"""
Localization templatetags unit tests
"""
import pytest

from django.test import override_settings
from django.core.exceptions import ImproperlyConfigured

from phac_aspc.django.helpers.templatetags.phac_aspc_wet import (
    jsdelivr,
    phac_aspc_wet_css,
    phac_aspc_wet_scripts,
    phac_aspc_wet_session_timeout_dialog,
)

urlpatterns = []


class Request:  # pylint: disable=too-few-public-methods
    """Mock request object"""

    user = None

    def __init__(self, user):
        self.user = user


class User:  # pylint: disable=too-few-public-methods
    """Mock user object"""

    is_authenticated = False

    def __init__(self, auth=False):
        self.is_authenticated = auth


@override_settings(WET_VERSION="test.testing", THEME_VERSION="theme.version")
def test_jsdelivr():
    """
    Test the get_language template tag returns the correct information
    """

    assert (
        jsdelivr("wet-boew", "test.js")
        == "https://cdn.jsdelivr.net/gh/wet-boew/wet-boew-dist@test.testing/test.js"
    )

    assert (
        jsdelivr("some-other", "test.js")
        == "https://cdn.jsdelivr.net/gh/wet-boew/some-other-dist@theme.version/test.js"
    )


@override_settings(WET_VERSION="a", THEME_VERSION="b")
def test_phac_aspc_wet_css():
    """Test CSS tag generator"""
    html = phac_aspc_wet_css()
    assert html == (
        '<link rel="stylesheet" '
        'href="https://cdn.jsdelivr.net/gh/wet-boew/themes-dist@b/'
        'GCWeb/css/theme.min.css">'
        "<noscript>"
        '<link rel="stylesheet" href="https://cdn.jsdelivr.net/gh/wet-boew/'
        'wet-boew-dist@a/wet-boew/css/noscript.min.css">'
        "</noscript>"
    )

    html_only_base = phac_aspc_wet_css(base_only=True)
    assert html_only_base == (
        '<link rel="stylesheet" '
        'href="phac_aspc_helpers/base.css">'
        "<noscript>"
        '<link rel="stylesheet" href="https://cdn.jsdelivr.net/gh/wet-boew/'
        'wet-boew-dist@a/wet-boew/css/noscript.min.css">'
        "</noscript>"
    )


@override_settings(WET_VERSION="a", THEME_VERSION="b")
def test_phac_aspc_wet_scripts():
    """Test script tag generator"""
    assert (
        phac_aspc_wet_scripts()
        == """
        
      <script
        src="https://ajax.googleapis.com/ajax/libs/jquery/2.2.4/jquery.min.js"
        integrity="sha384-rY/jv8mMhqDabXSo+UCggqKtdmBfd3qC2/KvyTDNQ6PcUJXaxK1tMepoQda4g5vB"
        crossorigin="anonymous"
      ></script>
        <script src="https://cdn.jsdelivr.net/gh/wet-boew/wet-boew-dist@a/wet-boew/js/wet-boew.min.js"></script>
        <script src="https://cdn.jsdelivr.net/gh/wet-boew/themes-dist@b/GCWeb/js/theme.min.js"></script>
    """
    )

    assert (
        phac_aspc_wet_scripts(include_jquery=False)
        == """
        
        <script src="https://cdn.jsdelivr.net/gh/wet-boew/wet-boew-dist@a/wet-boew/js/wet-boew.min.js"></script>
        <script src="https://cdn.jsdelivr.net/gh/wet-boew/themes-dist@b/GCWeb/js/theme.min.js"></script>
    """
    )


@override_settings(
    SESSION_COOKIE_AGE=60,
    ROOT_URLCONF="phac_aspc.django.helpers.urls",
    TEMPLATES=[
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "APP_DIRS": True,
        },
    ],
)
def test_phac_aspc_wet_session_timeout_dialog():
    """Test session timeout dialog template tag"""
    context_logged_in = {"request": Request(User(True))}
    context_logged_out = {"request": Request(User())}
    assert phac_aspc_wet_session_timeout_dialog(context_logged_out, "test_logout") == ""
    html = phac_aspc_wet_session_timeout_dialog(context_logged_in, "test_logout")
    assert "&quot;logouturl&quot;: &quot;test_logout&quot;" in html
    assert "&quot;inactivity&quot;: 48000.0" in html
    assert "&quot;reactionTime&quot;: 12000.0" in html
    assert "&quot;sessionalive&quot;: 60000" in html


@override_settings(
    SESSION_COOKIE_AGE=60,
    ROOT_URLCONF=__name__,
    TEMPLATES=[
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "APP_DIRS": True,
        },
    ],
)
def test_phac_aspc_wet_session_timeout_dialog_errors():
    """Test session timeout dialog template tag"""
    with pytest.raises(ImproperlyConfigured):
        phac_aspc_wet_session_timeout_dialog(
            {"request": Request(User(True))}, "test_logout"
        )



""" Make all templatetags available to things like Jinja"""
from .phac_aspc_localization import phac_aspc_localization_lang
from .phac_aspc_wet import (
    WET_CDN_ROOT,
    jsdelivr,
    phac_aspc_wet_css,
    phac_aspc_wet_scripts,
    phac_aspc_wet_session_timeout_dialog,
)
from .phac_aspc_auth import phac_aspc_auth_signin_microsoft_button

__all__ = [
    "phac_aspc_localization_lang",
    "WET_CDN_ROOT",
    "jsdelivr",
    "phac_aspc_wet_css",
    "phac_aspc_wet_scripts",
    "phac_aspc_wet_session_timeout_dialog",
    "phac_aspc_auth_signin_microsoft_button",
]



"""Recommended localization settings"""
from django.utils.translation import gettext_lazy as _

from .utils import global_from_env

LANGUAGES = (
    ("fr-ca", _("French")),
    ("en-ca", _("English")),
)

global_from_env(
    LANGUAGE_CODE=(str, "en-ca"),
)



"""
Localization templatetags unit tests
"""
from phac_aspc.django.helpers.templatetags.phac_aspc_localization import get_language
from phac_aspc.django.helpers.locale.language import locale_lang


def test_get_language():
    """
    Test the get_language template tag returns the correct information
    """
    with locale_lang("fr-ca"):
        assert get_language() == "fr-ca"

    with locale_lang("en-ca"):
        assert get_language() == "en-ca"
